@@ -1,10 +1,11 @@
 import json
 import logging
+import typing
 from contextlib import contextmanager
 from hashlib import sha1
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import List, Dict, Set, Tuple
+from typing import List, Dict, Set, Tuple, Any
 from warnings import warn
 from urllib.parse import urlparse
 
@@ -16,6 +17,7 @@ from newpyter.utils import remove_prefix, remove_suffix
 
 logger = logging.getLogger(__file__)
 
+FragmentType = Any
 
 def get_hash(content: bytes) -> str:
     """ Default way is to use SHA1 hashing """
@@ -51,7 +53,7 @@ class StorageContext:
         self.cache: Dict[str, bytes] = {}
         self.encryptor = encryptor
 
-    def store_fragment(self, fragment: 'json') -> str:
+    def store_fragment(self, fragment: FragmentType) -> str:
         assert isinstance(fragment, list)
         fragment = json.dumps(fragment).encode('utf-8')
         assert isinstance(fragment, bytes)
@@ -72,7 +74,7 @@ def output_for_missing_fragment(hash):
 
 class AbstractStorage:
     def __init__(self, encryptor: Encryptor, local_cache: Path):
-        self.url = None
+        self.url: str # should be assigned in implementations
         self.encryptor = encryptor
         self.local_cache = local_cache
         self.notebook_specific_caches = local_cache.joinpath('temp_folders')
@@ -108,10 +110,10 @@ class AbstractStorage:
     def _upload_fragments(self, for_upload: Dict[str, bytes]):
         raise NotImplementedError("Storing is not implemented")
 
-    def _download_fragments(self, hashes: List[str]) -> List[bytes]:
+    def _download_fragments(self, hashes: List[str]) -> typing.Iterable[bytes]:
         raise NotImplementedError("Loading is not implemented in storage")
 
-    def get_decrypted_fragments(self, hashes: List[str]) -> Dict[str, 'json']:
+    def get_decrypted_fragments(self, hashes: List[str]) -> Dict[str, FragmentType]:
         hash2encrypted_fragment = {}
         hashes_for_downloading = []
         for hash in set(hashes):
@@ -153,7 +155,7 @@ class MemoryMockStorage(AbstractStorage):
     def store_fragments(self, fragments: Dict[str, bytes]):
         return self.stored.update(fragments)
 
-    def get_decrypted_fragments(self, hashes: List[str]) -> Dict[str, 'json']:
+    def get_decrypted_fragments(self, hashes: List[str]) -> Dict[str, FragmentType]:
         if self.return_empty:
             return {hash: output_for_missing_fragment(hash) for hash in hashes}
         return {
@@ -174,7 +176,7 @@ class SSHStorage(AbstractStorage):
         ssh_path = remove_suffix(ssh_path, '/') + '/'
         self.url = ssh_path
         self.ssh_key = ssh_key
-        self.key_params = []
+        self.key_params: list = []
         if self.ssh_key is not None:
             self.key_params = ['-i', Path(self.ssh_key).expanduser()]
 
@@ -186,7 +188,7 @@ class SSHStorage(AbstractStorage):
                 # TODO use something better than scp
                 sh.scp(*self.key_params, '-r', Path(temp_dir).joinpath(hash), self.url + hash)
 
-    def _download_fragments(self, hashes: List[str]) -> List[bytes]:
+    def _download_fragments(self, hashes: List[str]) -> typing.Iterator[bytes]:
         for hash in hashes:
             path_on_this_machine = self.local_cache.joinpath(hash)
             if not path_on_this_machine.exists():
@@ -248,7 +250,8 @@ class S3Storage(AbstractStorage):
                     client.download_file(self.bucket, self.path_in_bucket + hash, str(path_on_this_machine))
                 except BaseException as _:
                     logger.info(f'did not locate {self.path_in_bucket + hash} in bucket {self.bucket}, prefixing with / now')
-                    # for a long time files were saved
+                    # in old protocol versions, file was saved with a leading slash (by mistake),
+                    # this isn't so since ~nov 2021, and after ~ apr 2023 this condition should be deleted
                     client.download_file(self.bucket, '/' + self.path_in_bucket + hash, str(path_on_this_machine))
                 # this hash is already there and will not be uploaded
                 uploaded_path_hashes.add((self.url, hash))
